package com.samelement.iot_notification;

import android.Manifest;
import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.PowerManager;
import android.provider.Settings;
import android.util.Base64;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.view.ContextThemeWrapper;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.PermissionChecker;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.preference.PreferenceManager;
import androidx.work.Constraints;
import androidx.work.ExistingWorkPolicy;
import androidx.work.NetworkType;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkManager;

import com.samelement.iot_notification.screen.IntentKey;
import com.samelement.iot_notification.screen.SimpleScannerActivity;
import com.samelement.iot_notification.service.AccessTokenRepository;
import com.samelement.iot_notification.service.SamService;
import com.samelement.iot_notification.service.worker.RestartServiceWorker;
import com.samelement.iot_notification.util.EncryptUtil;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.UUID;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * IotNotificationPlugin
 */
public class IotNotificationPlugin implements MethodCallHandler, FlutterPlugin,
        PluginRegistry.ActivityResultListener, ActivityAware, PluginRegistry.RequestPermissionsResultListener {
    private final String TAG = IotNotificationPlugin.class.getSimpleName();

    private Context context;
    private Activity activity;
    private MethodChannel channel;
    private Result scanResult;

    private static final String CHANNEL_NAME = "iot_notification";
    private static final int REQUEST_SCAN_CODE = 1175522;
    private static final int REQUEST_CAMERA_PERMISSION = 887433800;
    private static final int REQUEST_WRITE_STORAGE_PERMISSION = 839920831;

    static {
        System.loadLibrary("keys");
    }

    public native String getKey();
    public native String getIv();

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        IotNotificationPlugin instance = new IotNotificationPlugin();
        instance.channel = new MethodChannel(registrar.messenger(), CHANNEL_NAME);
        instance.context = registrar.context();
        instance.channel.setMethodCallHandler(instance);
    }


    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        channel = new MethodChannel(binding.getBinaryMessenger(), CHANNEL_NAME);
        context = binding.getApplicationContext();
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
        channel = null;
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        switch (call.method) {
            case "getKeys":
                List<String> keys = new ArrayList<>();
                keys.add(new String(Base64.decode(getKey(), Base64.DEFAULT)));
                keys.add(new String(Base64.decode(getIv(), Base64.DEFAULT)));
                result.success(keys);
                break;
            case "stopService":
                Intent serviceIntent = new Intent(context, SamService.class);
                context.stopService(serviceIntent);
                break;
            case "startService":
                if (call.hasArgument(IntentKey.KEY_USER_ACCESS_TOKEN)) {
                    String userAccessToken = call.argument(IntentKey.KEY_USER_ACCESS_TOKEN);
                    String userRefreshToken = call.argument(IntentKey.KEY_USER_REFRESH_TOKEN);
                    String devAccessToken = call.argument(IntentKey.KEY_DEV_ACCESS_TOKEN);
                    String devRefreshToken = call.argument(IntentKey.KEY_DEV_REFRESH_TOKEN);
                    String mqttClientIdentifier = call.argument(IntentKey.KEY_MQTT_IDENTIFIER);
                    String brokerAddress = call.argument(IntentKey.KEY_MQTT_BROKER_ADDRESS);
                    String brokerProtocol = call.argument(IntentKey.KEY_MQTT_BROKER_PROTOCOL);
                    int brokerPort = call.argument(IntentKey.KEY_MQTT_BROKER_PORT);
                    String apiHost = call.argument(IntentKey.KEY_API_HOST);
                    String cafile = call.argument(IntentKey.KEY_CAFILE);

                    SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(context);
                    if (preferences.contains(IntentKey.KEY_MQTT_IDENTIFIER)) {
                        mqttClientIdentifier = preferences.getString(IntentKey.KEY_MQTT_IDENTIFIER, UUID.randomUUID().toString());
                    }

                    int notifId = createId();
                    if (preferences.contains(IntentKey.KEY_NOTIF_ID)) {
                        notifId = preferences.getInt(IntentKey.KEY_NOTIF_ID, 90909090);
                    }

                    preferences.edit()
                            .putString(IntentKey.KEY_MQTT_IDENTIFIER, mqttClientIdentifier)
                            .putString(IntentKey.KEY_MQTT_BROKER_ADDRESS, brokerAddress)
                            .putString(IntentKey.KEY_MQTT_BROKER_PROTOCOL, brokerProtocol)
                            .putInt(IntentKey.KEY_MQTT_BROKER_PORT, brokerPort)
                            .putString(IntentKey.KEY_API_HOST, apiHost)
                            .putString(IntentKey.KEY_CAFILE, cafile)
                            .putInt(IntentKey.KEY_NOTIF_ID, notifId).apply();

                    String encodedUserAccessToken = EncryptUtil.getInstance(context).encrypt(userAccessToken);
                    String encodedUserRefreshToken = EncryptUtil.getInstance(context).encrypt(userRefreshToken);

                    String encodedDevAccessToken = EncryptUtil.getInstance(context).encrypt(devAccessToken);
                    String encodedDevRefreshToken = EncryptUtil.getInstance(context).encrypt(devRefreshToken);

                    // cek access token lama
                    String oldEncodedUserAccessToken = preferences.getString(IntentKey.KEY_USER_ACCESS_TOKEN, null);
                    if (oldEncodedUserAccessToken != null) {
                        if (encodedUserAccessToken != oldEncodedUserAccessToken) {
                            AccessTokenRepository.getInstance(context).updateTokenExpired();
                        }
                    }

                    preferences.edit()
                            .putString(IntentKey.KEY_USER_ACCESS_TOKEN, encodedUserAccessToken)
                            .putString(IntentKey.KEY_USER_REFRESH_TOKEN, encodedUserRefreshToken)
                            .putString(IntentKey.KEY_DEV_ACCESS_TOKEN, encodedDevAccessToken)
                            .putString(IntentKey.KEY_DEV_REFRESH_TOKEN, encodedDevRefreshToken)
                            .apply();

                    restartService();
                }

//                ActivityCompat.requestPermissions(activity, new String[]{
//                        Manifest.permission.WRITE_EXTERNAL_STORAGE
//                }, REQUEST_WRITE_STORAGE_PERMISSION);
                break;
            case "themeChanged":
                if (call.hasArgument(IntentKey.KEY_THEME_ID)) {
                    int themeId = call.argument(IntentKey.KEY_THEME_ID);
                    SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(context);
                    preferences.edit().putInt(IntentKey.KEY_THEME_ID, themeId).apply();
                }
                break;
            case "deleteAnalytic":
                if (call.hasArgument("data")) {
                    Intent intent = new Intent(IntentKey.FILTER_DEL_ANALYTIC);

                    String data = call.argument("data");
                    intent.putExtra("data", data);

                    LocalBroadcastManager.getInstance(context).sendBroadcast(intent);
                }
                break;
            case "addDevice":
                if (call.hasArgument("data")) {
                    Intent intent = new Intent(IntentKey.FILTER_ADD_DEVICE);

                    String data = call.argument("data");
                    intent.putExtra("data", data);

                    LocalBroadcastManager.getInstance(context).sendBroadcast(intent);
                }
                break;
            case "deleteDevice":
                if (call.hasArgument("data")) {
                    Intent intent = new Intent(IntentKey.FILTER_DEL_DEVICE);

                    String data = call.argument("data");
                    intent.putExtra("data", data);

                    LocalBroadcastManager.getInstance(context).sendBroadcast(intent);
                }
                break;
            case "updateDevice":
                if (call.hasArgument("data")) {
                    try {
                        Intent intent = new Intent(IntentKey.FILTER_UPD_DEVICE);

                        String data = call.argument("data");
                        intent.putExtra("data", data);
                        LocalBroadcastManager.getInstance(context).sendBroadcast(intent);
                    } catch (Exception e) {
                        System.out.println(e.getMessage());
                    }
                }
                break;
            case "addAnalytic":
            case "updateAnalytic":
                restartService();
                break;
            case "openPowerManager":
                try {
                    for (final Intent intent : POWERMANAGER_INTENTS) {
                        if (context.getPackageManager().resolveActivity(intent, PackageManager.MATCH_DEFAULT_ONLY) != null) {
                            final SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(context);
                            boolean powerManagerOpened = preferences.getBoolean(IntentKey.POWER_MANAGER_OPENED, false);

                            if (!powerManagerOpened) {
                                // show dialog to ask user action
                                new AlertDialog.Builder(new ContextThemeWrapper(context,R.style.myDialog))
                                        .setTitle("Enable AutoStart?")
                                        .setMessage("Please allow the application to always run in the background, else our services can't be accessed")

                                        // Specifying a listener allows you to take an action before dismissing the dialog.
                                        // The dialog is automatically dismissed when a dialog button is clicked.
                                        .setPositiveButton("Open settings", new DialogInterface.OnClickListener() {
                                            public void onClick(DialogInterface dialog, int which) {
                                                try {
                                                    preferences.edit().putBoolean(IntentKey.POWER_MANAGER_OPENED, true).apply();
                                                } catch (Exception e) {
                                                    Log.d(TAG, e.getMessage() == null ? "null" : e.getMessage());
                                                }
                                                context.startActivity(intent);
                                            }
                                        })
                                        // A null listener allows the button to dismiss the dialog and take no further action.
                                        .setNegativeButton("Ignore", null)
                                        .setIcon(android.R.drawable.ic_dialog_alert)
                                        .show();
                            }
                            break;
                        }
                    }
                } catch (Exception e) {
                    System.out.println(e.getMessage());
                    e.printStackTrace();
                }
                break;
            case "ignoreBatteryOptimization":
                try {
                    PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
                    if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
                        boolean isIgnoringBatteryOptimizations = pm.isIgnoringBatteryOptimizations(context.getPackageName());
                        if (!isIgnoringBatteryOptimizations) {
                            Intent intent = new Intent();
                            intent.setAction(Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS);
                            intent.setData(Uri.parse("package:" + context.getPackageName()));
                            // context.startActivityForResult(intent, MY_IGNORE_OPTIMIZATION_REQUEST);
                        }
                    }
                } catch (Exception e) {
                    System.out.println(e.getMessage());
                    e.printStackTrace();
                }
                break;
            case "scan":
                scanResult = result;

                if (!checkPermission(Manifest.permission.CAMERA)) {
                    ActivityCompat.requestPermissions(activity, new String[]{
                            Manifest.permission.CAMERA
                    }, REQUEST_CAMERA_PERMISSION);
                } else showScanner();
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void showScanner() {
        Intent intent = new Intent(context, SimpleScannerActivity.class);
        activity.startActivityForResult(intent, REQUEST_SCAN_CODE);
    }

    private boolean checkPermission(String permission) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            return ContextCompat.checkSelfPermission(context, permission) == PackageManager.PERMISSION_GRANTED;
        } else {
            return PermissionChecker.checkSelfPermission(context, permission) == PermissionChecker.PERMISSION_GRANTED;
        }
    }

    private void restartService() {
        try {
            WorkManager.getInstance(context).cancelUniqueWork("requestRestartServiceFromPlugin");

            Constraints constraints = new Constraints.Builder()
                    .setRequiredNetworkType(NetworkType.CONNECTED)
                    .build();

            OneTimeWorkRequest restartRequest = new OneTimeWorkRequest.Builder(RestartServiceWorker.class)
                    .setConstraints(constraints)
                    .build();

            WorkManager.getInstance(context).enqueueUniqueWork("requestRestartServiceFromPlugin", ExistingWorkPolicy.REPLACE, restartRequest);
        } catch (Exception e) {
            Log.d(TAG, "Error restart service.");
        }
    }

    private int createId() {
        Date now = new Date();
        return Integer.parseInt(new SimpleDateFormat("ddHHmmss", Locale.US).format(now));
    }

    public static final int MY_IGNORE_OPTIMIZATION_REQUEST = 5089;
    private static final Intent[] POWERMANAGER_INTENTS = {
            new Intent().setComponent(new ComponentName("com.miui.securitycenter", "com.miui.permcenter.autostart.AutoStartManagementActivity")),
            new Intent().setComponent(new ComponentName("com.letv.android.letvsafe", "com.letv.android.letvsafe.AutobootManageActivity")),
            new Intent().setComponent(new ComponentName("com.huawei.systemmanager", "com.huawei.systemmanager.startupmgr.ui.StartupNormalAppListActivity")),
            new Intent().setComponent(new ComponentName("com.huawei.systemmanager", "com.huawei.systemmanager.optimize.process.ProtectActivity")),
            new Intent().setComponent(new ComponentName("com.huawei.systemmanager", "com.huawei.systemmanager.appcontrol.activity.StartupAppControlActivity")),
            new Intent().setComponent(new ComponentName("com.coloros.safecenter", "com.coloros.safecenter.permission.startup.StartupAppListActivity")),
            new Intent().setComponent(new ComponentName("com.coloros.safecenter", "com.coloros.safecenter.startupapp.StartupAppListActivity")),
            new Intent().setComponent(new ComponentName("com.oppo.safe", "com.oppo.safe.permission.startup.StartupAppListActivity")),
            new Intent().setComponent(new ComponentName("com.iqoo.secure", "com.iqoo.secure.ui.phoneoptimize.AddWhiteListActivity")),
            new Intent().setComponent(new ComponentName("com.iqoo.secure", "com.iqoo.secure.ui.phoneoptimize.BgStartUpManager")),
            new Intent().setComponent(new ComponentName("com.vivo.permissionmanager", "com.vivo.permissionmanager.activity.BgStartUpManagerActivity")),
            new Intent().setComponent(new ComponentName("com.samsung.android.lool", "com.samsung.android.sm.ui.battery.BatteryActivity")),
            new Intent().setComponent(new ComponentName("com.htc.pitroad", "com.htc.pitroad.landingpage.activity.LandingPageActivity")),
            new Intent().setComponent(new ComponentName("com.asus.mobilemanager", "com.asus.mobilemanager.MainActivity"))
    };

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_SCAN_CODE) {
            if (resultCode == Activity.RESULT_OK) {
                if (data != null) {
                    scanResult.success(data.getStringExtra("result"));
                }
            }
        }

        return false;
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        binding.addActivityResultListener(this);
        binding.addRequestPermissionsResultListener(this);
        activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }

    @Override
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        if (requestCode == REQUEST_CAMERA_PERMISSION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                showScanner();
                return true;
            }
        }

        if (requestCode == REQUEST_WRITE_STORAGE_PERMISSION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                return true;
            }
        }

        return false;
    }
}
